@@ -258,29 +258,67 @@ class ProxyColumn(Proxy, DictMixin):
             yield k
 
     def iteritems(self):
+        count = 0
         oid = self.proxy.oid
         indexes = self.proxy.table.index
-        while True:
-            noid, result = self.session.getnext(oid)[0]
-            if noid <= oid:
-                raise StopIteration
-            oid = noid
-            if not((len(oid) >= len(self.proxy.oid) and
-                    oid[:len(self.proxy.oid)] == self.proxy.oid[:len(self.proxy.oid)])):
-                raise StopIteration
 
-            # oid should be turned into index
-            index = oid[len(self.proxy.oid):]
-            target = []
-            for x in indexes:
-                l, o = x.type.fromOid(x, tuple(index))
-                target.append(x.type(x, o))
-                index = index[l:]
-            if len(target) == 1:
-                # Should work most of the time
-                yield target[0], result
-            else:
-                yield tuple(target), result
+        while True:
+            try:
+                if self.session.bulk and self.session.use_bulk:
+                    results = self.session.getbulk(oid)
+                else:
+                    results = self.session.getnext(oid)
+            except snmp.SNMPEndOfMibView:
+                break
+
+            for noid, result in results:
+                if noid <= oid:
+                    noid = None
+                    break
+                oid = noid
+                if not((len(oid) >= len(self.proxy.oid) and
+                    oid[:len(self.proxy.oid)] == self.proxy.oid[:len(self.proxy.oid)])):
+                    noid = None
+                    break
+
+                # oid should be turned into index
+                index = oid[len(self.proxy.oid):]
+                target = []
+                for x in indexes:
+                    l, o = x.type.fromOid(x, tuple(index))
+                    target.append(x.type(x, o))
+                    index = index[l:]
+                count = count + 1
+                if len(target) == 1:
+                    # Should work most of the time
+                    yield target[0], result
+                else:
+                    yield tuple(target), result
+
+            if noid is None:
+                break
+
+        if count == 0:
+            # We did not find any element. Is it because the column is
+            # empty or because the column does not exist. We do a SNMP
+            # GET to know. If we get a "No such instance" exception,
+            # this means the column is empty. If we get "No such
+            # object", this means the column does not exist. We cannot
+            # make such a distinction with SNMPv1.
+            try:
+                self.session.get(self.proxy.oid)
+            except snmp.SNMPNoSuchInstance:
+                # OK, the set of result is really empty
+                raise StopIteration
+            except snmp.SNMPNoSuchName:
+                # SNMPv1, we don't know
+                pass
+            except snmp.SNMPNoSuchObject:
+                # The result is empty because the column is unknown
+                raise
+
+        raise StopIteration
+
 
 loaded = []
 def load(mibname):
